//
//  HTMLStoreParser.m
//  League Manager
//
//  Created by Michael Privat on 9/20/10.
//  Copyright (c) 2010 __MyCompanyName__. All rights reserved.
//

#import "HTMLStoreParser.h"
#import "HTMLStore.h"

@implementation HTMLStoreParser

@synthesize store, coordinator, nodes;

-(HTMLStoreParser*)initWithStore:(HTMLStore*)_store {
  self.store = _store;  
  
  coordinator = [store persistentStoreCoordinator];
  self.nodes = [NSMutableSet set];
  
  return self;
}

/*
 * OPENING tags
 */
- (void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName
                        namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qualifiedName
                        attributes:(NSDictionary *)attributeDict {
  
  if([elementName isEqualToString:@"tr"]) {
    propertyIndex = 0;
  }
  else if([elementName isEqualToString:@"td"]) {
    idList = [[NSMutableArray alloc] init];
  }

  value = nil;
}

/*
 * CLOSING tags
 */
- (void)parser:(NSXMLParser *)parser didEndElement:(NSString *)elementName
                        namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName {

  if([elementName isEqualToString:@"h1"]) {
    currentEntity = [[[coordinator managedObjectModel] entitiesByName] valueForKeyPath:value];
    attributes = [currentEntity attributesByName];
    relationships = [currentEntity relationshipsByName];
  }
  else if([elementName isEqualToString:@"th"]) {
    if(propertyNames == nil) {
      // First attribute is always id
      propertyNames = [[NSMutableArray alloc] init];
    }

    [propertyNames addObject:value];
  }
  else if([elementName isEqualToString:@"a"]) {
    // This can be one of two things: an anchor ending or a to-many item in which
    // case value won't be nil
    
    if(value != nil) {
      [idList addObject:value];
    }
  }
  else if([elementName isEqualToString:@"td"]) {    
    if(propertyIndex == 0) {
      NSManagedObjectID *oid = [store objectIDForEntity:currentEntity referenceObject:value];
      node = [store nodeForReferenceObject:value andObjectID:oid];    
    }
    else {
      NSString *propertyName = [propertyNames objectAtIndex:propertyIndex];
      if([attributes objectForKey:propertyName] != nil) {
        NSAttributeDescription *attributeDescription = [attributes objectForKey:propertyName];
        NSAttributeType type = [attributeDescription attributeType];
        
        // Default value to type string
        id dataValue = value;        
        if([(NSString*)dataValue isEqualToString:@"(null)"]) {
          // Skip to leave null
        }        
        else if ((type == NSInteger16AttributeType) || (type == NSInteger32AttributeType) || (type == NSInteger64AttributeType)) {
          dataValue = [NSNumber numberWithInteger:[dataValue integerValue]];
        }
        else if (type == NSDecimalAttributeType) {
          dataValue = [NSDecimalNumber decimalNumberWithString:dataValue];
        }
        else if (type == NSDoubleAttributeType) {
          dataValue = [NSNumber numberWithDouble:[dataValue doubleValue]];
        }
        else if (type == NSFloatAttributeType) {
          dataValue = [NSNumber numberWithFloat:[dataValue floatValue]];
        }
        else if (type == NSBooleanAttributeType) {
          dataValue = [NSNumber numberWithBool:[dataValue intValue]];
        }
        else if (type == NSDateAttributeType) {
          NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
          [formatter setDateFormat:@"yyyy-MM-dd HH:mm:ss ZZZ"];
          dataValue = [formatter dateFromString:dataValue];
          [formatter release];            
        }
        else if (type == NSBinaryDataAttributeType) {
          NSLog(@"Binary type not supported");
        }
        
        [node setValue:dataValue forKey:propertyName];
      }
      else if([relationships objectForKey:propertyName] != nil) {
        NSRelationshipDescription *relationship = [relationships objectForKey:propertyName];

        if([relationship isToMany]) {
          NSMutableSet* set = [NSMutableSet set];
          for(NSString *fKey in idList) {
            if(fKey != nil && ![fKey isEqualToString:@"(null)"]) {
              NSManagedObjectID *oid = [store objectIDForEntity:[relationship destinationEntity] referenceObject:fKey];
              NSAtomicStoreCacheNode *destinationNode = [store nodeForReferenceObject:fKey andObjectID:oid];
              [set addObject:destinationNode];
            }                            
          }
          [node setValue:set forKey:propertyName];
        }
        else {  // to-One
          NSString* fKey = [idList count] > 0 ? [idList objectAtIndex:0] : nil;
          if(fKey != nil && ![fKey isEqualToString:@"(null)"]) {
            NSManagedObjectID *oid = [store objectIDForEntity:[relationship destinationEntity] referenceObject:fKey];
            NSAtomicStoreCacheNode *destinationNode = [store nodeForReferenceObject:fKey andObjectID:oid];
            [node setValue:destinationNode forKey:propertyName];
          }
        }
        
        if(idList != nil) {
          [idList release];
          idList = nil;
        }
      }
    }
    
    propertyIndex++;
  }
  else if([elementName isEqualToString:@"tr"]) {
    if(node != nil) {
      [nodes addObject: node];
      node = nil;
    }
  }
  else if([elementName isEqualToString:@"table"]) {
    if(propertyNames != nil) {
      [propertyNames release];
      propertyNames = nil;
    }
  }
                   
  value = nil;
}

- (void)parser:(NSXMLParser *)parser foundCharacters:(NSString *)characters {  
  if(!value) {
    value = [[NSMutableString alloc] initWithString:characters];
  }
  else {
    [value appendString:characters];
  }
}

- (void)dealloc {
  [self.nodes release];
  
  [super dealloc];
}
@end
