//
//  PlayerListViewController.m
//  League Manager
//
//  Created by Rob Warner on 8/24/10.
//  Copyright (c) 2010 Availity, LLC. All rights reserved.
//

#import "PlayerListViewController.h"
#import "RootViewController.h"
#import "PlayerViewController.h"

@implementation PlayerListViewController

@synthesize team;
@synthesize rootController;

- (id)initWithRootController:(RootViewController *)aRootController team:(NSManagedObject *)aTeam {
  if ((self = [super init])) {
    self.rootController = aRootController;
    self.team = aTeam;
  }
  return self;
}

#pragma mark -
#pragma mark View lifecycle

- (void)viewDidLoad {
  [super viewDidLoad];
  
  self.title = @"Players";
  
  UIBarButtonItem *addButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemAdd target:self action:@selector(showPlayerView)];
  self.navigationItem.rightBarButtonItem = addButton;
  [addButton release];
}

- (void)showPlayerView {
  PlayerViewController *playerViewController = [[PlayerViewController alloc] initWithRootController:rootController team:team player:nil];
  [self presentModalViewController:playerViewController animated:YES];
  [playerViewController release];  
}

- (void)viewWillAppear:(BOOL)animated {
  [super viewWillAppear:animated];
  [self.tableView reloadData];
}

#pragma mark -
#pragma mark Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
  return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
  return [(NSSet *)[team valueForKey:@"players"] count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
  static NSString *CellIdentifier = @"PlayerCell";
  
  UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
  if (cell == nil) {
    cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:CellIdentifier] autorelease];
  }
  
  NSManagedObject *player = [[self sortPlayers] objectAtIndex:indexPath.row];
  cell.textLabel.text = [NSString stringWithFormat:@"%@ %@", [[player valueForKey:@"firstName"] description],
                         [[player valueForKey:@"lastName"] description]];
  cell.detailTextLabel.text = [[player valueForKey:@"email"] description];
  return cell;
}

- (NSArray *)sortPlayers {
  NSSortDescriptor *sortLastNameDescriptor = [[[NSSortDescriptor alloc] initWithKey:@"lastName" ascending:YES]
                                              autorelease];
  NSArray *sortDescriptors = [NSArray arrayWithObjects:sortLastNameDescriptor, nil];
  return [[(NSSet *)[team valueForKey:@"players"] allObjects] sortedArrayUsingDescriptors:sortDescriptors];
}

#pragma mark -
#pragma mark Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
  NSManagedObject *player = [[self sortPlayers] objectAtIndex:indexPath.row];
  PlayerViewController *playerViewController = [[PlayerViewController alloc] initWithRootController:rootController team:team player:player];
  [self presentModalViewController:playerViewController animated:YES];
  [playerViewController release];
}

#pragma mark -
#pragma mark Memory management

- (void)didReceiveMemoryWarning {
  [super didReceiveMemoryWarning];
}

- (void)viewDidUnload {
}

- (void)dealloc {
  [team release];
  [rootController release];
  [super dealloc];
}

@end

